@@ -2,27 +2,24 @@
  * playlist-files.c
  * Copyright 2010-2011 John Lindgren
  *
- * This file is part of Audacious.
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are met:
  *
- * Audacious is free software: you can redistribute it and/or modify it under
- * the terms of the GNU General Public License as published by the Free Software
- * Foundation, version 2 or version 3 of the License.
+ * 1. Redistributions of source code must retain the above copyright notice,
+ *    this list of conditions, and the following disclaimer.
  *
- * Audacious is distributed in the hope that it will be useful, but WITHOUT ANY
- * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
- * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
+ * 2. Redistributions in binary form must reproduce the above copyright notice,
+ *    this list of conditions, and the following disclaimer in the documentation
+ *    provided with the distribution.
  *
- * You should have received a copy of the GNU General Public License along with
- * Audacious. If not, see <http://www.gnu.org/licenses/>.
- *
- * The Audacious team does not consider modular code linking to Audacious or
- * using our public API to be a derived work.
+ * This software is provided "as is" and without any warranty, express or
+ * implied. In no event shall the authors be liable for any damages arising from
+ * the use of this software.
  */
 
 #include <glib.h>
 #include <libaudcore/audstrings.h>
 
-#include "config.h"
 #include "debug.h"
 #include "i18n.h"
 #include "misc.h"
@@ -30,23 +27,23 @@
 #include "plugin.h"
 #include "plugins.h"
 
-static const char * get_extension (const char * filename)
+static PluginHandle * get_plugin_silent (const char * filename)
 {
-    const char * ext;
-    uri_parse (filename, NULL, & ext, NULL, NULL);
-    return (ext && ext[0] == '.') ? ext + 1 : NULL;
+    char buf[32];
+    if (! uri_get_extension (filename, buf, sizeof buf))
+        return NULL;
+
+    return playlist_plugin_for_extension (buf);
 }
 
 bool_t filename_is_playlist (const char * filename)
 {
-    const char * ext = get_extension (filename);
-    return (ext && playlist_plugin_for_extension (ext)) ? TRUE : FALSE;
+    return (get_plugin_silent (filename) != NULL);
 }
 
 static PluginHandle * get_plugin (const char * filename, bool_t saving)
 {
-    const char * ext = get_extension (filename);
-    PluginHandle * plugin = ext ? playlist_plugin_for_extension (ext) : NULL;
+    PluginHandle * plugin = get_plugin_silent (filename);
 
     if (! plugin)
     {
